@@ -36,10 +36,6 @@ BUILD_TYPE_RELEASE = "release"
 BUILD_TYPE_PROFILING = "profiling"
 
 TOOLCHAIN_DEFAULT = "default"
-TOOLCHAIN_VS16_MSVC = "vs16_msvc"
-TOOLCHAIN_VS16_LLVM13 = "vs16_llvm13"
-TOOLCHAIN_VS17_MSVC = "vs17_msvc"
-TOOLCHAIN_VS17_LLVM13 = "vs17_llvm13"
 
 OS_WINDOWS = "windows"
 
@@ -151,11 +147,7 @@ class ToolchainSettings(ABC):
 
 class ToolchainSettingsProvider(ABC):
     @abstractmethod
-    def get_toolchain_settings(self, toolchain_name) -> ToolchainSettings:
-        pass
-
-    @abstractmethod
-    def get_toolchains(self) -> set[str]:
+    def get_toolchain_settings(self, toolchain_name: str) -> ToolchainSettings:
         pass
 
 
