@@ -13,6 +13,10 @@ import re
 required_conan_version = ">=1.43 <=1.62"
 
 
+THIRD_PARTY_MANIFEST_FILENAME = "third_party_gen.json"
+TOOLCHAIN_MANIFEST_FILENAME = "toolchain_gen.json"
+
+
 def _sanitize_path(path: str):
     return os.path.normpath(path)
 
@@ -60,20 +64,9 @@ class ToolchainDiscoveryWindows:
     def dump_to_json(self):
         return json.dumps(self.toolchains, indent=4)
 
-    # Settings allowed for users.
-    # Whenever a new allowed setting is added, a detection mechanism must be added for it below.
-    _USER_SUPPORTED_SETTINGS = {
-        "compiler": {
-            "Visual Studio": {
-                "version": ["15", "16", "17"]
-            },
-            "clang": {
-                "version": ["13"]
-            }
-        },
-        "arch": ["x86_64"],
-        "build_type": ["Debug", "Release"]
-    }
+    _TARGET_ARCHITECTURES = [
+        "x86_64",
+    ]
 
     _OS_WINDOWS_KEY = "win"
 
@@ -94,29 +87,43 @@ class ToolchainDiscoveryWindows:
     def discover_toolchains(self, conanfile):
         # TODO: Get rid of conanfile argument - we discover everything we can, disregarding conanfile settings.
         # The argument is required for tools.vcvars_dict, even though we override the important options.
-        # Replace tools.vcvars_dir with tools.vswhere.
-        compilers = self._USER_SUPPORTED_SETTINGS["compiler"]
 
-        vs_settings = compilers["Visual Studio"]
+        #
+        # Detect Visual Studio installations.
+        #
         valid_vc_vars = []
+        vs_major_versions = []
 
-        for vs_version in vs_settings["version"]:
+        vs_installations = tools.vswhere()
+        for vs_installation in vs_installations:
+            try:
+                ver = tools.Version(vs_installation["installationVersion"])
+                major = str(ver.major)
+                if major not in vs_major_versions:
+                    vs_major_versions.append(major)
+            except KeyError:
+                raise ToolchainDiscoveryError("Unable to detect Visual Studio installations - internal error.")
+
+        for vs_version in vs_major_versions:
             # VS uses different toolchains for targeting different architectures.
-            for arch in self._USER_SUPPORTED_SETTINGS["arch"]:
+            for arch in self._TARGET_ARCHITECTURES:
                 try:
                     vc_vars = tools.vcvars_dict(conanfile, arch=arch, compiler_version=vs_version, filter_known_paths=True)
                 except ConanException as e:
-                    conanfile.output.info(f"Visual Studio version {vs_version} for arch {arch} -- not found.")
+                    raise ToolchainDiscoveryError("Unable to detect MSVC metadata - internal error.")
                 else:
-                    conanfile.output.info(f"Visual Studio version {vs_version} for arch {arch} -- found.")
+                    vc_tools_version = vc_vars["VCToolsVersion"]
+                    vscmd_ver = vc_vars["VSCMD_VER"]
+                    conanfile.output.info(f"MSVC -- found version {vc_tools_version} (VS: {vscmd_ver}) for arch {arch}.")
                     valid_vc_vars.append((vs_version, arch, vc_vars))
 
         for vs_version, arch, vc_vars in valid_vc_vars:
             self._add_toolchain(self._OS_WINDOWS_KEY, f"vs{vs_version}_msvc", arch,
                                 self._generate_toolchain_manifest_vs_msvc(vc_vars))
 
-        clang_settings = compilers["clang"]
-
+        #
+        # Detect clang installation.
+        #
         clang_path = tools.which("clang")
         if clang_path is not None:
 
@@ -140,21 +147,24 @@ class ToolchainDiscoveryWindows:
             major = match.group(1)
             minor = match.group(2)
             patch = match.group(3)
-            conanfile.output.info(f"Compiler 'clang' -- found version {major}.{minor}.{patch}")
+            conanfile.output.info(f"Compiler 'clang' -- found version {major}.{minor}.{patch}.")
 
-            supported_versions = clang_settings["version"]
-            if major in supported_versions:
-                # Go up twice
-                llvm_root_path = os.path.split(os.path.split(clang_path)[0])[0]
-                for vs_version, arch, vc_vars in valid_vc_vars:
-                    self._add_toolchain(self._OS_WINDOWS_KEY, f"vs{vs_version}_llvm{major}", arch,
-                                        self._generate_toolchain_manifest_vs_llvm(vc_vars, llvm_root_path,
-                                                                                  SemVer(major, minor, patch)))
-            else:
-                vers = ", ".join(supported_versions)
-                conanfile.output.info(f"Compiler 'clang' -- supported major versions are [{vers}], skipping")
+            # Go up twice
+            llvm_root_path = os.path.split(os.path.split(clang_path)[0])[0]
+            for vs_version, arch, vc_vars in valid_vc_vars:
+                self._add_toolchain(self._OS_WINDOWS_KEY, f"vs{vs_version}_llvm{major}", arch,
+                                    self._generate_toolchain_manifest_vs_llvm(vc_vars, llvm_root_path,
+                                                                              SemVer(major, minor, patch)))
         else:
-            conanfile.output.info(f"Compiler 'clang' -- not found")
+            conanfile.output.info(f"Compiler 'clang' -- not found.")
+
+    @staticmethod
+    def gen_vswhere_json():
+        return json.dumps(tools.vswhere(), indent=4)
+
+    @staticmethod
+    def gen_vcvars_json(conanfile):
+        return json.dumps(tools.vcvars_dict(conanfile, only_diff=True), indent=4)
 
     @staticmethod
     def _generate_toolchain_manifest_vs_msvc(vc_vars):
@@ -168,6 +178,7 @@ class ToolchainDiscoveryWindows:
 
         toolchain = {
             "compiler_family": "msvc",
+            "version": vc_vars["VCToolsVersion"],  # eg: 14.38.33130
             "compiler_path": os.path.join(vc_bin_path, "cl.exe"),
             "librarian_path": os.path.join(vc_bin_path, "lib.exe"),
             "linker_path": os.path.join(vc_bin_path, "link.exe"),
@@ -203,6 +214,7 @@ class ToolchainDiscoveryWindows:
     def _generate_toolchain_manifest_vs_llvm(vc_vars, llvm_root_path, llvm_semver):
         toolchain = {
             "compiler_family": "clang",
+            "version": str(llvm_semver),
             "compiler_path": os.path.join(llvm_root_path, "bin", "clang++.exe"),
             "librarian_path": os.path.join(llvm_root_path, "bin", "llvm-ar.exe"),
             "linker_path": os.path.join(llvm_root_path, "bin", "lld-link.exe"),  # Must use lld-link on Windows.
@@ -215,12 +227,24 @@ class ToolchainDiscoveryWindows:
         # compiling a simple cpp file.
 
         # The clang include path must be first in the list.
-        include_dirs = [os.path.join(llvm_root_path, "lib", "clang", str(llvm_semver), "include")]
+        clang_include_path = os.path.join(llvm_root_path, "lib", "clang", str(llvm_semver), "include")
+        if not os.path.exists(clang_include_path):
+            clang_include_path = os.path.join(llvm_root_path, "lib", "clang", str(llvm_semver.major), "include")
+        if not os.path.exists(clang_include_path):
+            raise RuntimeError("Could not determine clang include path.")
+
+        clang_lib_dir_path =  os.path.join(llvm_root_path, "lib", "clang", str(llvm_semver), "lib", "windows")
+        if not os.path.exists(clang_lib_dir_path):
+            clang_lib_dir_path =  os.path.join(llvm_root_path, "lib", "clang", str(llvm_semver.major), "lib", "windows")
+        if not os.path.exists(clang_include_path):
+            raise RuntimeError("Could not determine clang lib dir path.")
+
+        include_dirs = [clang_include_path]
         include_dirs.extend(_sanitize_paths(vc_vars["INCLUDE"]))
 
         lib_dirs = _sanitize_paths(vc_vars["LIB"])
         # The clang lib dir path must be last on the list.
-        lib_dirs.append(os.path.join(llvm_root_path, "lib", "clang", str(llvm_semver), "lib", "windows"))
+        lib_dirs.append(clang_lib_dir_path)
 
         toolchain["toolchain_include_dirs"] = include_dirs
         toolchain["toolchain_lib_dirs"] = lib_dirs
@@ -239,9 +263,6 @@ def create_toolchain_discovery():
 # noinspection PyBroadException
 class RapidGenerator(Generator):
 
-    THIRD_PARTY_MANIFEST_FILENAME = "third_party_gen.json"
-    TOOLCHAIN_MANIFEST_FILENAME = "toolchain_gen.json"
-
     # noinspection PyPropertyDefinition
     @property
     def filename(self):
@@ -258,12 +279,14 @@ class RapidGenerator(Generator):
                 td.discover_toolchains(self.conanfile)
 
                 return {
-                    self.THIRD_PARTY_MANIFEST_FILENAME: self._generate_third_party_manifest_json(),
-                    self.TOOLCHAIN_MANIFEST_FILENAME: td.dump_to_json()
+                    THIRD_PARTY_MANIFEST_FILENAME: self._generate_third_party_manifest_json(),
+                    TOOLCHAIN_MANIFEST_FILENAME: td.dump_to_json(),
+                    # "debug_vswhere_output.json": td.gen_vswhere_json(),
+                    # "debug_vcvars.json": td.gen_vcvars_json(self.conanfile),
                 }
             else:
                 return {
-                    self.THIRD_PARTY_MANIFEST_FILENAME: self._generate_third_party_manifest_json(),
+                    THIRD_PARTY_MANIFEST_FILENAME: self._generate_third_party_manifest_json(),
                 }
 
         except Exception:
@@ -339,5 +362,9 @@ class RapidGenerator(Generator):
 
 class Pkg(ConanFile):
     name = "rapidbuild"
-    version = "0.5.1"
+    version = "0.7.0"
     license = "MIT"
+
+
+def _validate_version(pkg_version: str):
+    assert (Pkg.version == pkg_version)
\ No newline at end of file
