@@ -1,6 +1,8 @@
+import re
 from contextlib import contextmanager
 import fs
 from typing import Any
+from conans import tools as conan_tools
 
 from .target_graph import (
     TargetImplementation,
@@ -10,7 +12,8 @@ from .workspace_def import (
 )
 from . import base
 
-from .compiler_base import CompilerArgFormatter, CompilerSettings
+from .compiler_base import CompilerArgFormatter, CompilerSettings, LanguageStandard
+from .compiler_msvc import CompilerArgFormatterMSVC
 from .linker_base import LinkerArgFormatter, LinkerSettings
 from .toolchain import (
     CompilerFamily,
@@ -98,6 +101,12 @@ class BFFEmitter:
         else:
             self._emit_line(".{} = {}".format(name, self._format_value(value, should_split_str=should_split_str)))
 
+    @contextmanager
+    def unity(self, name: str):
+        self._begin_function("Unity", name)
+        yield
+        self._end_function()
+
     @contextmanager
     def library(self, name: str):
         self._begin_function("Library", name)
@@ -260,12 +269,13 @@ def _build_compiler_definitions_args(
 
 def _build_compiler_compiland_args(
         arg_formatter: CompilerArgFormatter,
+        language: LanguageStandard,
         output_dir_path: str, output_base_name: str,
         pch_header_path: str | None, pch_output_path: str | None,
 ):
     args = []
     args += arg_formatter.format_beginning_args()
-    args += arg_formatter.format_object_compilation_args()
+    args += arg_formatter.format_object_compilation_args(language)
     if pch_header_path is not None and pch_output_path is not None:
         args += arg_formatter.format_pch_usage_args(pch_header_path, pch_output_path)
     args += arg_formatter.format_debug_info_args(output_dir_path, output_base_name)
@@ -313,13 +323,15 @@ class BuildScriptEmitterFastbuild(base.BuildScriptEmitter):
     ALL_TARGETS_TARGET_NAME = "rapid_all"
     SLN_TARGET_NAME = "rapid_sln"
     CLEAN_TARGET_NAME = "rapid_clean"
-    TARGET_WORKING_DIR_NAME = "data"
 
     HEADER_EXTENSIONS = ["h", "hh", "hxx", "hpp"]
     SOURCE_EXTENSIONS = ["c", "cc", "cxx", "cpp"]
 
     PCH_OUTPUT_EXT = "pch"
 
+    def __init__(self, data_dir_abs_path: str):
+        self.data_dir_abs_path = data_dir_abs_path
+
     def filename(self):
         return "fbuild.bff"
 
@@ -331,8 +343,6 @@ class BuildScriptEmitterFastbuild(base.BuildScriptEmitter):
                  targets_impls
                  ) -> str:
 
-        assert (set(toolchains_manifest.keys()).issubset(toolchains_settings_provider.get_toolchains()))
-
         intermediate_dir_abs_path = fs.path.join(build_dir_abs_path, "intermediate")
         deploy_dir_abs_path = fs.path.join(build_dir_abs_path, "bin")
 
@@ -340,10 +350,14 @@ class BuildScriptEmitterFastbuild(base.BuildScriptEmitter):
         out_bff.once()
 
         self._emit_compilers(out_bff, toolchains_manifest)
-        self._emit_compilation_targets(out_bff, intermediate_dir_abs_path, deploy_dir_abs_path, toolchains_manifest,
-                                       toolchains_settings_provider, build_types, targets_names, targets_impls)
-        self._emit_vs_solution_targets(out_bff, wks_name, source_dir_abs_path, configure_dir_abs_path, deploy_dir_abs_path,
-                                       toolchains_manifest, toolchains_settings_provider, build_types, targets_names, targets_impls)
+        self._emit_compilation_targets(
+            out_bff, intermediate_dir_abs_path, deploy_dir_abs_path, toolchains_manifest,
+            toolchains_settings_provider, build_types, targets_names, targets_impls
+        )
+        self._emit_vs_solution_targets(
+            out_bff, wks_name, source_dir_abs_path, configure_dir_abs_path, deploy_dir_abs_path, self.data_dir_abs_path,
+            toolchains_manifest, toolchains_settings_provider, build_types, targets_names, targets_impls
+        )
 
         with out_bff.remove_dir(self.CLEAN_TARGET_NAME):
             out_bff.emit_var_assignment("RemovePaths", build_dir_abs_path)
@@ -414,7 +428,8 @@ class BuildScriptEmitterFastbuild(base.BuildScriptEmitter):
             toolchains_settings_provider: base.ToolchainSettingsProvider,
             build_types: list[str],
             targets_names: list[str],
-            targets_impls: dict[str, TargetImplementation]
+            targets_impls: dict[str, TargetImplementation],
+            enable_unity: bool = True,
     ):
 
         all_nodes: list[str] = []
@@ -469,20 +484,48 @@ class BuildScriptEmitterFastbuild(base.BuildScriptEmitter):
                     target_kind = target_impl.kind
                     target_output_dir_path = fs.path.join(intermediate_dir_abs_path, target_node_name)
 
+                    compiler_input_dir = target_impl.source_dir
+
+                    pch_header_file, pch_source_file = self._search_pch_files_in_dir(compiler_input_dir)
+                    pch_output_file = fs.path.join(target_output_dir_path,
+                                                   f"{target_node_name}.{self.PCH_OUTPUT_EXT}")
+
+                    if enable_unity:
+                        unity_node_name = self._format_unity_node_name(target_node_name)
+                        with out_bff.unity(unity_node_name):
+                            out_bff.emit_var_assignment("UnityInputPath", compiler_input_dir)
+                            out_bff.emit_var_assignment("UnityInputPathRecurse", True)
+                            unity_input_pattern = [f"*.{ext}" for ext in self.SOURCE_EXTENSIONS]
+                            out_bff.emit_var_assignment("UnityInputPattern", unity_input_pattern)
+
+                            if pch_header_file is not None:
+                                out_bff.emit_var_assignment("UnityPCH", fs.path.basename(pch_header_file))
+
+                            unity_output_dir = fs.path.join(target_output_dir_path, "unity")
+                            out_bff.emit_var_assignment("UnityOutputPath", unity_output_dir)
+
                     def emit_bff_compiler_args():
-                        out_bff.emit_var_assignment("Compiler", self._format_compiler_node_name(toolchain_name))
+                        #
+                        # Compiler input
+                        #
+                        if enable_unity:
+                            unity_node_name = self._format_unity_node_name(target_node_name)
+                            out_bff.emit_var_assignment("CompilerInputUnity", unity_node_name)
+                        else:
+                            out_bff.emit_var_assignment("CompilerInputPath", compiler_input_dir)
+                            out_bff.emit_var_assignment("CompilerInputPathRecurse", True)
+                            compiler_input_pattern = [f"*.{ext}" for ext in self.SOURCE_EXTENSIONS]
+                            out_bff.emit_var_assignment("CompilerInputPattern", compiler_input_pattern)
 
-                        compiler_input_dir = target_impl.source_dir
-                        out_bff.emit_var_assignment("CompilerInputPath", compiler_input_dir)
-
-                        out_bff.emit_var_assignment("CompilerInputPathRecurse", True)
-
-                        compiler_input_pattern = [f"*.{ext}" for ext in self.SOURCE_EXTENSIONS]
-                        out_bff.emit_var_assignment("CompilerInputPattern", compiler_input_pattern)
-
-                        compiler_output_dir = fs.path.join(target_output_dir_path, target_name)
+                        #
+                        # Compiler output
+                        #
+                        compiler_output_dir = fs.path.join(target_output_dir_path, "obj")
                         out_bff.emit_var_assignment("CompilerOutputPath", compiler_output_dir)
 
+                        #
+                        # Compiler settings
+                        #
                         # TODO: Per-target compiler settings.
                         per_target_compiler_settings = CompilerSettings()
                         def combine(lhs, rhs):
@@ -501,21 +544,22 @@ class BuildScriptEmitterFastbuild(base.BuildScriptEmitter):
                                                                         target_impl.definitions)
 
 
-                        pch_header_file, pch_source_file = self._search_pch_files_in_dir(compiler_input_dir)
-                        pch_output_file = fs.path.join(target_output_dir_path,
-                                                       f"{target_node_name}.{self.PCH_OUTPUT_EXT}")
-
                         compiland_args = _build_compiler_compiland_args(
                             compiler_arg_formatter,
+                            target_compiler_settings.lang_std,
                             target_output_dir_path, target_node_name,
                             pch_header_file, pch_output_file
                         )
                         compiland_args += common_args
 
 
+                        out_bff.emit_var_assignment("Compiler", self._format_compiler_node_name(toolchain_name))
                         out_bff.emit_var_assignment("CompilerOptions", " ".join(compiland_args),
                                                     should_split_str=True)
 
+                        #
+                        # PCH
+                        #
                         if pch_header_file is not None and pch_source_file is not None:
                             out_bff.line_break()
 
@@ -532,19 +576,19 @@ class BuildScriptEmitterFastbuild(base.BuildScriptEmitter):
                             out_bff.emit_var_assignment("PCHOptions", " ".join(pch_args), should_split_str=True)
                             out_bff.line_break()
 
+                    def emit_bff_librarian_args():
+                        librarian_output_file = fs.path.join(target_output_dir_path,
+                                                             self._format_target_filename(target_node_name,
+                                                                                          target_kind))
+
+                        out_bff.emit_var_assignment("Librarian", toolchain_def["librarian_path"])
+                        out_bff.emit_var_assignment("LibrarianOutput", librarian_output_file)
+                        out_bff.emit_var_assignment("LibrarianOptions",
+                                                    " ".join(librarian_arg_formatter.format_input_output_args()),
+                                                    should_split_str=True)
+
                     if target_kind == TargetKind.STATIC_LIB:
 
-                        def emit_bff_librarian_args():
-                            librarian_output_file = fs.path.join(target_output_dir_path,
-                                                                 self._format_target_filename(target_node_name,
-                                                                                              target_kind))
-
-                            out_bff.emit_var_assignment("Librarian", toolchain_def["librarian_path"])
-                            out_bff.emit_var_assignment("LibrarianOutput", librarian_output_file)
-                            out_bff.emit_var_assignment("LibrarianOptions",
-                                                        " ".join(librarian_arg_formatter.format_input_output_args()),
-                                                        should_split_str=True)
-
                         with out_bff.library(make_node_name):
                             out_bff.emit_var_assignment("Hidden", True)
                             emit_bff_compiler_args()
@@ -659,28 +703,41 @@ class BuildScriptEmitterFastbuild(base.BuildScriptEmitter):
             source_dir_abs_path: str,
             configure_dir_abs_path: str,
             deploy_dir_abs_path: str,
+            data_dir_abs_path: str,
             toolchains_manifest: dict[str, Any],
             toolchains_settings_provider: base.ToolchainSettingsProvider,
             build_types: list[str],
             targets_names: list[str],
             targets_impls: dict[str, TargetImplementation]
     ):
+        target_platform = "x64"
 
-        bff_path = fs.path.join(f"{configure_dir_abs_path}", self.filename())
-        vs_dir_abs_path = fs.path.join(configure_dir_abs_path, "vs")
+        # Determine latest MSVC version.
+        latest_msvc_semver = conan_tools.Version("0.0.0")
+        for toolchain in toolchains_manifest.values():
+            if toolchain["compiler_family"] == "msvc":
+                semver = conan_tools.Version(toolchain["version"])
+                if semver > latest_msvc_semver:
+                    latest_msvc_semver = semver
+
+        if latest_msvc_semver.major == 0:
+            return
+
+        # eg: 14.31 -> v143
+        latest_msvc_toolset = f"v{latest_msvc_semver.major}{(int(latest_msvc_semver.minor) % 100) // 10}"
 
-        latest_msvc_toolset = None
         latest_vs_version = None
-        if base.TOOLCHAIN_VS17_MSVC in toolchains_manifest.keys():
-            latest_msvc_toolset = "v143"
-            latest_vs_version = "17.0"
-        elif base.TOOLCHAIN_VS16_MSVC in toolchains_manifest.keys():
-            latest_msvc_toolset = "v142"
-            latest_vs_version = "16.0"
+        if 10 <= int(latest_msvc_semver.minor) < 20:
+            latest_vs_version = "15.0"  # Visual Studio 2017
+        elif 20 <= int(latest_msvc_semver.minor) < 30:
+            latest_vs_version = "16.0"  # Visual Studio 2019
+        elif 30 <= int(latest_msvc_semver.minor):
+            latest_vs_version = "17.0"  # Visual Studio 2022
         else:
             return
 
-        lang_standard = "c++20"
+        bff_path = fs.path.join(f"{configure_dir_abs_path}", self.filename())
+        vs_dir_abs_path = fs.path.join(configure_dir_abs_path, "vs")
 
         # Fastbuild can deduce these automatically based of "Target", manual spec overrides it.
         # Currently, our "Target" is pointing to an alias, which points to a deployment step,
@@ -721,7 +778,7 @@ class BuildScriptEmitterFastbuild(base.BuildScriptEmitter):
                         out_bff._emit_line(".cfg = [")
                         out_bff._indent()
 
-                        out_bff.emit_var_assignment("Platform", "x64")
+                        out_bff.emit_var_assignment("Platform", target_platform)
                         out_bff.emit_var_assignment("PlatformToolset", latest_msvc_toolset)
                         out_bff.emit_var_assignment("Config", f"{toolchain_name}-{build_type}")
                         # out_bff.emit_var_assignment("Config", "Debug")
@@ -731,11 +788,12 @@ class BuildScriptEmitterFastbuild(base.BuildScriptEmitter):
                                                     f"fbuild -ide -config \"{bff_path}\" {target_node_name}")
                         out_bff.emit_var_assignment("ProjectRebuildCommand",
                                                     f"fbuild -ide -config \"{bff_path}\" -clean {target_node_name}")
-                        # ProjectCleanCommand
-                        out_bff.emit_var_assignment("LocalDebuggerCommand",
-                                                    fs.path.join(build_type_deploy_dir_path, f"{target_node_name}.exe"))
-                        out_bff.emit_var_assignment("LocalDebuggerWorkingDirectory",
-                                                    fs.path.join(source_dir_abs_path, target_name, self.TARGET_WORKING_DIR_NAME))
+                        # TODO: ProjectCleanCommand
+
+                        if target_impl.kind == TargetKind.EXECUTABLE:
+                            out_bff.emit_var_assignment("LocalDebuggerCommand",
+                                                        fs.path.join(build_type_deploy_dir_path, f"{target_node_name}.exe"))
+                            out_bff.emit_var_assignment("LocalDebuggerWorkingDirectory", data_dir_abs_path)
 
                         out_bff.line_break()
 
@@ -766,8 +824,11 @@ class BuildScriptEmitterFastbuild(base.BuildScriptEmitter):
 
                             out_bff.emit_var_assignment("PreprocessorDefinitions", definitions_intellisense)
 
-                            # TODO: generate AdditionalOptions from compiler options - standard and warning settings
-                            out_bff.emit_var_assignment("AdditionalOptions", f"/std:{lang_standard}")
+                            lang_std = toolchain_settings.get_compiler_settings(build_type).lang_std
+                            lang_std_arg = CompilerArgFormatterMSVC.format_language_std(lang_std)
+
+                            # TODO: populate additional options with warning settings
+                            out_bff.emit_var_assignment("AdditionalOptions", f"{lang_std_arg[0]}")
 
                         out_bff._dedent()
                         out_bff._emit_line("]")
@@ -793,7 +854,7 @@ class BuildScriptEmitterFastbuild(base.BuildScriptEmitter):
                     out_bff._emit_line(".cfg = [")
                     out_bff._indent()
 
-                    out_bff.emit_var_assignment("Platform", "x64")
+                    out_bff.emit_var_assignment("Platform", target_platform)
                     out_bff.emit_var_assignment("PlatformToolset", latest_msvc_toolset)
                     out_bff.emit_var_assignment("Config", f"{toolchain_name}-{build_type}")
                     # out_bff.emit_var_assignment("Config", "Debug")
@@ -803,11 +864,9 @@ class BuildScriptEmitterFastbuild(base.BuildScriptEmitter):
                                                 f"fbuild -ide -config \"{bff_path}\" {target_node_name}")
                     out_bff.emit_var_assignment("ProjectRebuildCommand",
                                                 f"fbuild -ide -config \"{bff_path}\" -clean {target_node_name}")
-                    # ProjectCleanCommand
+                    # #TODO: ProjectCleanCommand
                     out_bff.emit_var_assignment("LocalDebuggerCommand", "")
 
-                    out_bff.emit_var_assignment("AdditionalOptions", f"/std:{lang_standard}")
-
                     out_bff._dedent()
                     out_bff._emit_line("]")
                     out_bff._emit_line(".ProjectConfigs + .cfg")
@@ -906,6 +965,10 @@ class BuildScriptEmitterFastbuild(base.BuildScriptEmitter):
     def _format_obj_node_name(target_node_name: str):
         return f"obj-{target_node_name}"
 
+    @staticmethod
+    def _format_unity_node_name(target_node_name: str):
+        return f"unity-{target_node_name}"
+
     @staticmethod
     def _format_make_node_name(target_node_name: str):
         return f"make-{target_node_name}"
