@@ -26,7 +26,7 @@ def _validate_third_party_manifest(manifest):
     return True
 
 
-def _validate_toolchains_manifest(manifest, supported_toolchains):
+def _validate_toolchains_manifest(manifest):
     if not isinstance(manifest, dict):
         return False
     if len(manifest.keys()) == 0:
@@ -69,12 +69,6 @@ class Workspace:
             base.BUILD_TYPE_RELEASE,
             base.BUILD_TYPE_PROFILING
         ]
-        self.supported_toolchains = [
-            base.TOOLCHAIN_VS16_MSVC,
-            base.TOOLCHAIN_VS16_LLVM13,
-            base.TOOLCHAIN_VS17_MSVC,
-            base.TOOLCHAIN_VS17_LLVM13,
-        ]
 
         self.logger = logger
         self.system_config_provider = system_config_provider
@@ -171,26 +165,14 @@ class Workspace:
 
         self.toolchains_manifest = self.system_config_provider.get_toolchains_manifest()
 
-        if not _validate_toolchains_manifest(self.toolchains_manifest, self.supported_toolchains):
+        if not _validate_toolchains_manifest(self.toolchains_manifest):
             self.logger.log_error("Provided toolchains manifest is invalid.")
             return False
 
         manifest_toolchains = self.toolchains_manifest.keys()
-        if not set(manifest_toolchains).issubset(self.supported_toolchains):
-            self.logger.log_error("Manifest toolchains must be a subset of supported toolchains.")
-            self.logger.log_error(f"Toolchains in manifest: {sorted(manifest_toolchains)}")
-            self.logger.log_error(f"Supported toolchains: {sorted(self.supported_toolchains)}")
-            return False
-
-        settings_toolchains = self.toolchain_settings_provider.get_toolchains()
-        if not set(manifest_toolchains).issubset(settings_toolchains):
-            self.logger.log_error("Settings must be provided for all toolchains from the toolchain manifest.")
-            self.logger.log_error(f"Manifest toolchains: {sorted(list(manifest_toolchains))}")
-            self.logger.log_error(f"Toolchains for which settings were provided : {sorted(list(settings_toolchains))}")
-            return False
 
         ok = True
-        for toolchain in self.toolchain_settings_provider.get_toolchains():
+        for toolchain in manifest_toolchains:
             settings = self.toolchain_settings_provider.get_toolchain_settings(toolchain)
             if settings.get_build_types() != set(self.build_types):
                 self.logger.log_error("Settings for all supported build types must be provided.")
@@ -243,7 +225,7 @@ class Workspace:
         return True
 
 
-def open_workspace(wks_dir: str, *, name=None, dev_mode=False) -> Workspace:
+def open_workspace(wks_dir: str, *, name=None, dev_mode=False, data_dir=None) -> Workspace:
     logger = base.LoggerDefault()
 
     wks_fs = OSFS(os.path.abspath(wks_dir))
@@ -265,12 +247,18 @@ def open_workspace(wks_dir: str, *, name=None, dev_mode=False) -> Workspace:
 
     toolchain_settings_provider = ToolchainSettingsProviderDefault(wks_fs, logger)
 
+    if data_dir is not None:
+        data_dir_abs_path = os.path.abspath(data_dir)
+        assert os.path.exists(data_dir_abs_path)
+    else:
+        data_dir_abs_path = ""
+
     wks = Workspace(
         name=wks_name,
         wks_fs=wks_fs,
         logger=logger,
         system_config_provider=system_config_provider,
         toolchain_settings_provider=toolchain_settings_provider,
-        build_script_emitter=BuildScriptEmitterFastbuild()
+        build_script_emitter=BuildScriptEmitterFastbuild(data_dir_abs_path)
     )
     return wks
